STATE_DATA = [
    {'id': 'test_id_1',
     'title': "Show All Tickets",
     'show_in_all_tickets': '1',
     'show_in_my_tickets': '0',
     },

    {'id': 'test_id_2',
     'title': "Show My Tickets",
     'show_in_all_tickets': '0',
     'show_in_my_tickets': '1',
     },

    {'id': '',
     'title': "Test Id State",
     'show_in_all_tickets': '1',
     'show_in_my_tickets': '1',
     }]


AREA_DATA = [
    {'id': u'test_id_1',
     'title': u'Internet',
     },

    {'id': u'test_id_2',
     'title': u'Intranet',
     },

    {'id': u'',
     'title': u'Test Id Area',
     }]


RELEASE_DATA = [
    {'id': u'version-1',
     'title': u'Version 1',
     },

    {'id': u'version-2',
     'title': u'Version 2',
     },

    {'id': u'',
     'title': u'Test Id Release',
     }]


PRIORITY_DATA = [
    {'id': u'test_id_1',
     'title': u'High',
     },

    {'id': u'test_id_2',
     'title': u'Low',
     },

    {'id': u'',
     'title': u'Test Id Priority',
     }]


RESPONSIBLE_DATA = [
    {'id': u'testuser',
     'title': u'Testuser',
     },

    {'id': u'admin',
     'title': u'Admin',
     }]
