.response-adder dl.collapsible dd.collapsibleContent {
    padding: 1em 0 1em 1em;
}
.response-actions {
    display:block;
    padding-top:2em;
}
.response-reply {
    margin-bottom: 2em;
    border: 1px dashed #454545;
    padding: 1em;
}
.response-reply .response-actions form {
    float: left;
    margin-right: 1em;
}
.issue-attachment{
    margin-top: 2em;
}

table.ticketbox-overview {
    width: 100%;
}

table.ticketbox-overview td{
    vertical-align: top;
}

#addResponse {
  margin-top: 2em;
  margin-bottom: 1em;
}

#addResponse > .collapsibleHeader {
  font-size: 100%;
  padding-left: 0;
  background: none;
  margin-top: -0.5em;
}

body.template-atct_edit #datagridwidget-table-availableStates tr th:first-child,
body.template-atct_edit #datagridwidget-table-availableStates tr td:first-child,
body.template-atct_edit #datagridwidget-table-availableReleases tr th:first-child,
body.template-atct_edit #datagridwidget-table-availableReleases tr td:first-child,
body.template-atct_edit #datagridwidget-table-availablePriorities tr th:first-child,
body.template-atct_edit #datagridwidget-table-availablePriorities tr td:first-child,
body.template-atct_edit #datagridwidget-table-availableAreas tr th:first-child,
body.template-atct_edit #datagridwidget-table-availableAreas tr td:first-child,
body.template-atct_edit #datagridwidget-table-availableVarieties tr th:first-child,
body.template-atct_edit #datagridwidget-table-availableVarieties tr td:first-child {
    /* do not display the ids from the datagrid-widget */
    display:none;
    width:0px;
}

/* Hide adable types menu on tickets */
body.template-ticket_view.portaltype-ticket #plone-contentmenu-factories{
    display:none;
}